#ifndef GENERATOR_FRAMEDATA_HH
#define GENERATOR_FRAMEDATA_HH

#include "DataSource.hh"
#include "Dacc.hh"
#include <list>

namespace generator {

  /**  FrameData allows the user to inject pre-recorded data into the 
    *  generation streams. 
    *
    *  <i>Usage note:</i> The data source sample rate must be set to the 
    *  same rate as the sample rate defined in the frame data.
    *  @author John Zweizig
    *  @version 1.1; last modified April 13, 2004
    */
  class FrameData : public DataSource {
  public:
      using DataSource::setParameter;

  public:
      /** Frame data constructor.
        */
      FrameData(void);

      /**  Parameter constructor.
        *  @param files   A list of files to be searched for data.
	*  @param channel name of channel to be fetched.
        */
      FrameData(const char* files, const char* channel);

      /**  Pre-recorded data source destructor.
       */
      ~FrameData(void);

      /**  Make an identical copy of a frame data source.
        */
      FrameData* clone(void) const;

      /**  Return the data source type.
        */
      const char* getDataType(void) const;

      /**  Specify the name of the channel to be read in.
        */
      void setChannel(const std::string& name);

      /**  Specify the list of files to be searched for appropriate data.
        */
      void setFileList(const std::string& name);

      /**  Specify the value of a parameter
        */
      virtual void setParameter(const std::string& name, 
				const std::string& val);

  private:  // private data types
      /**  The data input class maintains a list of files to be searched by 
        *  a frame data source. The input data structures may be shared by 
	*  several FrameData data sources for efficient access of multiple 
	*  data channels.
        *  @memo Data input class.
	*/
      class input {
      public:
	/**  Construct a data input object and optionally set the file list 
	  *  parameter.
	  */
	input(const char* fl=0);

	/**  Destroy the data input object.
	 */
	~input(void);

	/**  Get the File list specifier.
	  */
	const char* getFileList(void) const;

	/**  Set the file list.
	  */
	void setFileList(const std::string& fList);

	/**  Get the specified data string.
	  */
	void getData(const char* ch, const Time& t0, Interval Dt, 
		     int N, float* data);

	/**  Return a referece to the internal data accessor.
	  */
	Dacc& refDacc(void);
	const Dacc& refDacc(void) const;

      private:
	std::string mFileList;
	Dacc        mIn;
      };
      typedef std::list<input> InpList;
      typedef InpList::iterator  inp_iter;

  private:
      /**  Get a data series for the specified Time interval.
        */
      void getSeries(const Time& t0, Interval dT, int N, float* data);

      /**  Search the input accessor list for one covering the correct data 
        *  files.
	*/
      inp_iter findInput(const std::string& filelist);

      /**  Add a new input accessor to the list.
	*/
      inp_iter addInput(const std::string& filelist);

  private:  // private data
      static InpList mInputList;
      std::string  mChannel;
      inp_iter mInput;
  };


  //====================================  In-line FrameData methods.
  inline const char*
  FrameData::getDataType(void) const {
      return "FrameData";
  }

  //====================================  In-line Framedata::input methods.
  inline Dacc& 
  FrameData::input::refDacc(void) {
      return mIn;
  }

  inline const Dacc& 
  FrameData::input::refDacc(void) const {
      return mIn;
  }

  inline const char* 
  FrameData::input::getFileList(void) const {
      return mFileList.c_str();
  }

} // namespace generator

#endif  // GENERATOR_FRAMEDATA_HH
