#ifndef GENERATOR_SINDATA_HH
#define GENERATOR_SINDATA_HH

#include "generator/DataSource.hh"

namespace generator {

  /**  SineData is a sinusoid data generator. The generated waveform is:
    *  \begin{verbatim}
          x(t)  = A * sin(2*pi*F*(t-to) + Phi)
       \end{verbatim}
    *  where the generator Parameters are:
    *  \begin{tabular}{|l|c|l|}
    *  #A#   & - & Amplitude \\
    *  #F#   & - & Sinusoid frequency (in Hz) \\
    *  #Phi# & 0 & Phase at #t=to# (in degrees) \\
    *  \end{tabular} 
    *  @version 1.1; last modified February 23, 2004
    *  @author  John Zweizig
    */
  class SineData : public DataSource {
  public:
      /**  Default constructor.
        */
      SineData(void);

      /**  Parameter constructor.
        *  @param Amp  Sinusoid amplitude.
	*  @param Freq Sinusoid frequency in Hz.
	*  @param Phi  Sinusoid phase at #t=to#.
        */
      SineData(double Amp, double Freq, double Phi=0.0);

      /**  SineData Destructor.
        */
      ~SineData(void);

      /**  Make an exact copy of a SineData constructor.
        *  @return Pointer to copied object.
        */
      SineData* clone(void) const;

      /**  Return a constant string with the data source type name ("Sine").
	*  @return constant pointer to the character string "Sine".
	*/
      const char* getDataType(void) const;
  private:

      /**  Generate the data.
        */
      void getSeries(const Time& t0, Interval dT, int N, float* data);
  private:
      Time mT0;
  };

  //====================================  Inline methods
  inline const char*
  SineData::getDataType(void) const {
      return "Sine";
  }

} // namespace generator

#endif  // GENERATOR_SINGAUSS_HH
