#include "findStuff.hh"
#include "ParseLine.hh"
#include <iostream>
#include <fstream>
#include <cstdlib>
#ifndef __CINT__
#include "TSeries.hh"
#include "TrigRslt.hh"

#ifndef BIGSTUFF_CC
#define PIDCVSHDR "$Header: /ldcg_server/common/repository_gds/gds/Monitors/bigStuff/findStuff.cc,v 1.2 2003/04/11 21:35:37 jzweizig Exp $"
#define PIDTITLE "bigStuff_find"
#include "ProcIdent.hh"

EXECDAT(findStuff);
#endif   //BIGSTUFF_CC
#endif   //__CINT__

#ifndef __NO_NAMESPACE
using namespace std;
#endif


//======================================  Constructor
findStuff::findStuff(int argc,const char *argv[])
  : DatEnv(argc,argv),mStride(1.0),mWait(5),mSize(6.0),mOSC(getDacc()) {
  if (Debug() > 2) cout << "Starting findStuff..." << endl;
  const char *cfile="findStuff.conf";
  const char *oscfile="findStuff_osc.conf";
  const char *offfile=0;

  // Note to self:  Don't use finish() in the Constructor as it must complete
  // constructor before exiting.
  for (int i=1; i<argc; i++) {
    if (!strcmp("-conf", argv[i])) {
      if (chkNextArg(i,argv,argc)) cfile = argv[++i];
      else {
	cerr << "Couldn't find configuration file"<<endl;
	exit(Usage(argv[0])+1);
      }
    } else if (!strcmp("-osc", argv[i])) {
      if (chkNextArg(i,argv,argc)) oscfile = argv[++i];
      else {
	cerr << "Couldn't find OSC file"<<endl;
	exit(Usage(argv[0])+1);
      }
    } else if (!strcmp("-offline", argv[i])) {
      if (chkNextArg(i,argv,argc)) offfile = argv[++i];
      else {
	cerr << "Couldn't find file containing offline directories."<<endl;
	exit(Usage(argv[0])+1);
      }
    } else if (!strcmp("-stride", argv[i])) {
      if (chkNextArg(i,argv,argc)) mStride = atof(argv[++i]);
      else exit(Usage(argv[0])+1);
    } else if (!strcmp("-size", argv[i])) {
      if (chkNextArg(i,argv,argc)) mSize = atof(argv[++i]);
      else exit(Usage(argv[0])+1);
    } else if (!strcmp("-wait", argv[i])) {
      if (chkNextArg(i,argv,argc)) mWait = atoi(argv[++i]);
      else exit(Usage(argv[0])+1);
    } else if (!strcmp("-h",argv[i])|| !strcmp("--help",argv[i])) {
      exit(Usage(argv[0]) - Help());
    } else if (isDatEnvArg(argv[i])) {i++;
    } else exit(Usage(argv[0]));
  }

  //=====================  Check for offline files
  if (offfile) {
    std::ifstream offstr(offfile);
    if (!offstr) {
      cerr << "Couldn't open file containing offline directories."<<endl;
      exit(Usage(argv[0])+2);
    }
    char mLine[1024];
    while (!offstr.eof()) {
      offstr.getline(mLine, sizeof(mLine));
      getDacc().addFile(mLine);
    }
  }

  //=====================  Read in the configuration file
  mConfig = cfile;
  mOSC.readConfig(oscfile);
  ReadConfig();

  //=====================  Connect the channels to Dacc & set up variables
  Reset();
}


//====================================  Read the config file
void findStuff::ReadConfig(void) {
  if (Debug() > 2) cout<<"Reached ReadConfig(void)"<<endl;
  //=====================  Set up the line parser
  ParseLine lp(mConfig.c_str());
  if (Debug()) lp.setLog(cout);
  int nLine(0);
  for (int nArg=lp.getLine(); nArg>=0; nArg=lp.getLine()) {
    nLine++;
    //---------  Empty command line
    if (!lp[0]) {
      continue;
    //---------  Add a channel for monitoring
    } else if (nArg >= 7) {
      if (Debug()) cout<<"Adding channel: "<<lp[0]<<endl;
      mChans.push_back(findChannelStat(std::string(lp[0]),atof(lp[1]),
				       atof(lp[2]),atof(lp[3]),atof(lp[4]),
				       atol(lp[5]),atol(lp[6]) ));
      cout << mChans.back().getName()<<" thresholds\tmax (in counts): "
	   << mChans.back().getMax()<<",\tmin (in counts): "
	   << mChans.back().getMin()<<endl;
    //---------  Invalid statement
    } else {
      cerr<<"Invalid command in "<<mConfig<<", line "<<nLine<<": "
	  <<lp[0]<<endl;
    }
  }
}

//======================================  The data crunching routine
void findStuff::ProcessData(void) {
  if (mOSC.satisfied("Mode_Cleaner_locked")) {
    if (Debug() > 2)  cout << "Processing Data" << endl;
    //-----------  Set the timestamps
    mCurrent = Time((unsigned long)mTS[0]->getStartTime().totalS());
    //=====================  Check if there's a gap in data; 
    //=====================   if so, add it to mSkips
    if (! Almost((mPrevious + mTS[0]->getInterval()),mCurrent) || 
	!mTS[0]->isGood()) {
      mSkips.push_back(pair<Time,Interval>
		       (mPrevious,Interval(mCurrent - mPrevious)));
    }
    //=====================  Looks for "bigStuff" (data outside range)
    for (unsigned int i(0); i < mChans.size(); i++) {
      // Start of a trigger is here
      if ((mTS[i]->getMaximum() > mChans[i].max1) || 
	  (mTS[i]->getMinimum() < mChans[i].min1)   ) {
	mChans[i].startTime = mCurrent;
	mChans[i].wait = mWait;
      } else {
	// go through a wait cycle after an event before triggering
	if (mChans[i].wait > 0) mChans[i].wait--;
	// trigger is sent at the end of a cycle
	else if (mChans[i].wait == 0) {
	  trig::TrigRslt trigData("bigStuff",mChans[i].name.c_str());
	  trigData.setIfos(string(mTS[i]->getName()).substr(0,2).c_str());
	  trigData.setTime(mChans[i].startTime);
	  trigData.setPriority(trig::p_warn);
	  trigData.setDuration(Interval(mCurrent - mChans[i].startTime));
	  trigData.setDisposition(trig::d_metaDB);
	  cout << "Sending trigger for channel "<<mChans[i].name
	       <<", starting "<< mChans[i].startTime.getS()<<" for "
	       <<double(Interval(mCurrent - mChans[i].startTime))
	       <<" seconds."<<endl;
	  //	int trigerr=sendTrigger(trigData);
	  //	if (!trigerr) cerr <<"error sending trigger="<<trigerr<<endl;
	  mChans[i].wait = -1;
	  // the only other option, hopefully, is that mChans[i].wait == -1
	} else {
	  mChans[i].wait = -1;	// Just for good measure
	}
      }
    }
    mPrevious = mCurrent;
  }
}

//======================================  Resets the channel access and parameters
void findStuff::Reset(void) {
  if (Debug()>2) cout<<"Resetting channels and parameters."<<endl;
  //=====================  Connect the channels to Dacc
  mTS = new TSeries*[mChans.size()];
  for (unsigned int i(0); i < mChans.size(); i++) {
    mTS[i] = 0;
    if (!getDacc().isChannelRead(mChans[i].name.c_str())) {
      getDacc().addChannel(mChans[i].name.c_str(),1,&mTS[i]);
    } else cerr <<"Cannot find channel "<<mChans[i].name<<endl;
  }

  //=====================  Set up the rest of the variables
  getDacc().setStride(mStride);
  getDacc().fillData(mStride);
  mPrevious = mTS[0]->getStartTime();
  for (unsigned int i(0); i < mChans.size(); i++) 
    mTS[i]->setName(mChans[i].name.c_str());
}

//======================================  What to do on an interrupt signal
void findStuff::Attention(void) {
  if (Debug()>2) cout <<"User interrupt-- going to attention."<<endl;
  delete[] mTS;
  for (unsigned int i(0); i<mChans.size(); i++) 
    getDacc().rmChannel(mChans[i].name.c_str());
  mChans.clear();
  ReadConfig();
  cout << "Configuration file updated-- reaccessing data."<<endl;
  Reset();
}

//======================================  Destructor
findStuff::~findStuff(void) {
  delete[] mTS;
  if (Debug() > 2) cout << "findStuff is finished." << endl;
}

//======================================  Checks that next argument isn't an option
bool findStuff::chkNextArg(int i, const char* argv[], const int argc) {
  if (++i < argc) {
    std::string args(argv[i]);
    if (args[0] != '-') return true;
  } // Note: Doesn't check next arg for number value
  return false;
}

//======================================  Prints out a usage statement
int findStuff::Usage(const char* progName) {
  cerr << "Usage: "<< progName<<" [-conf <file>] [-osc <file>] "
       << "[-offline <file>] [-wait <sec>] [-stride <sec>] [-h | --help]" 
       << endl;
  return 1;
}

//======================================  Prints out a Help statement (not helpful)
int findStuff::Help(void) {
  cerr <<"findStuff finds when channel data falls outside an acceptable range\n"
       <<"defined by getStuff.\nOptions include:\n"
       <<"  -conf <file>   -- The configuration file to use as input \n"
       <<"  -wait <sec>    -- The # of seconds to wait after an event \n"
       <<"  -stride <sec>  -- The # of seconds to process at a time \n"
       <<"  -h or --help   -- Generates this help text and exits"<<endl;
  return 1;
}


//======================================  Reset the channel's threshold limits
void findChannelStat::setLimits(float Limitmin, float Limitmax) {
  if (Limitmax == 0.0) Limitmax = Limitmin;
  setMax(Limitmax);
  setMin(Limitmin);
  int chkLimit(checkLimits());
  //=====================  Check that set limits are appropriate
  if (chkLimit) {
    //===================  Check the maximum limit
    if (chkLimit >= 16) {
      cerr <<"Error:Maximum for "<<name<<" ";
      if ((chkLimit & 64)) { // OR (chkLimit && 16)) {
      	cerr <<"falls outside of ADC range. Decreasing limits by 20%."<<endl;
	Limitmax *= .8;
      } else if ((chkLimit & 128)) { // OR (chkLimit && 32)) {
	cerr <<"falls within given range of values. Increasing limits by 20%."<<endl;
	Limitmax *= 1.2;
      } else cerr <<"falls in unknown category.  Please review."<<endl; 
   }
    //===================  Check the minimum limit
    if (chkLimit & 15) {
      cerr <<"Error:Minimum for "<<name<<" ";
      if ((chkLimit & 8)) { // OR (chkLimit && 2)) {
      	cerr <<"falls outside of ADC range. Decreasing limits by 20%."<<endl;
	Limitmin *= .8;
      } else if ((chkLimit & 4)) { // OR (chkLimit && 1)) {
	cerr <<"falls below given range of values.  Increasing limits by 20%."<<endl;
	Limitmin *= 1.2;
      } else cerr <<"falls in unknown category.  Please review."<<endl; 
    }
    setLimits(Limitmin,Limitmax);
  }
}


//======================================  Set the maximum limit
void findChannelStat::setMax(float Limitmax) {
  max1 = (long int) (mean + Limitmax * sdev);
  max2 = (long int) (mean + Limitmax * adev);
}

//======================================  Set the minimum limit
void findChannelStat::setMin(float Limitmin) {
  min1 = (long int) (mean - Limitmin * sdev);
  min2 = (long int) (mean - Limitmin * adev);
}

//======================================  Check that our limits are sensible
// (they should fall inside the ADC counts); Return 8-bit flag: 2 bits ea for 
// max1,max2,min1,min2.  For each, 0=just right, 1=too high, 2=too low
int findChannelStat::checkLimits(void) {
  int chkFlag(0);
  //=====================  First check that limits > observed daily range
  if (max1 < refMax) chkFlag |= 128;// 2-0-0-0 = 128
  //  if (max2 < refMax) chkFlag |= 32; // 0-2-0-0 = 32
  if (min1 > refMin) chkFlag |= 4;  // 0-0-1-0 = 4
  //  if (min2 > refMin) chkFlag |= 1;  // 0-0-0-1 = 1
  long int limitADC(32767);
  //=====================  Next check if the data is from an RS-232 port or not
  if (abs(mean) > limitADC) {cout <<chkFlag<<endl; return chkFlag;}
  //=====================  Finally check that limits are within the ADC range
  if (max1 > limitADC) chkFlag |= 64;// 1-0-0-0 = 64
  //  if (max2 > limitADC) chkFlag |= 16;// 0-1-0-0 = 16
  limitADC = 0 - limitADC;
  if (min1 < limitADC) chkFlag |= 8; // 0-0-2-0 = 8
  //  if (min2 < limitADC) chkFlag |= 2; // 0-0-0-2 = 2
  cout <<chkFlag<<endl;
  return chkFlag; 
}

